@@ -19,9 +19,9 @@
 
 """Provides the interface to the AT-SPI Registry."""
 
-__id__        = "$Id: atspi.py,v 1.51 2006/12/12 03:40:49 lmonsanto Exp $"
-__version__   = "$Revision: 1.51 $"
-__date__      = "$Date: 2006/12/12 03:40:49 $"
+__id__        = "$Id: atspi.py 1852 2006-12-21 13:27:19Z wwalker $"
+__version__   = "$Revision: 1852 $"
+__date__      = "$Date: 2006-12-21 08:27:19 -0500 (Thu, 21 Dec 2006) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -1431,7 +1431,7 @@ def __printDesktops():
         for j in range(0, desktop.childCount):
             app = desktop.getChildAtIndex(j)
             print "    App %d: name=%s role=%s" \
-                  % (j, app.name, app.role)
+                  % (j, app.name, app.getRoleName())
 
 def __notifyEvent(event):
         print event.type, event.source.name, \
