@@ -20,9 +20,9 @@
 """Provides support for defining keybindings and matching them to input
 events."""
 
-__id__        = "$Id: keybindings.py,v 1.12 2006/11/17 02:28:33 wwalker Exp $"
-__version__   = "$Revision: 1.12 $"
-__date__      = "$Date: 2006/11/17 02:28:33 $"
+__id__        = "$Id: keybindings.py 1818 2006-12-13 01:21:31Z wwalker $"
+__version__   = "$Revision: 1818 $"
+__date__      = "$Date: 2006-12-12 20:21:31 -0500 (Tue, 12 Dec 2006) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -113,8 +113,8 @@ def getModifierNames(mods):
         text += _("Super") + "+"
     if mods & (1 << atspi.Accessibility.MODIFIER_META2):
         text += _("Meta2") + "+"
-    if mods & (1 << atspi.Accessibility.MODIFIER_META):
-        text += _("Meta") + "+"
+    #if mods & (1 << atspi.Accessibility.MODIFIER_META):
+    #    text += _("Meta") + "+"
     if mods & (1 << atspi.Accessibility.MODIFIER_ALT):
         text += _("Alt_L") + "+"
     if mods & (1 << atspi.Accessibility.MODIFIER_CONTROL):
@@ -190,11 +190,20 @@ class KeyBindings:
             if keyBinding == self.keyBindings[i]:
                 del self.keyBindings[i]
 
+    def removeByHandler(self, handler):
+        """Removes the given KeyBinding instance from this set of keybindings.
+        """
+        i = len(self.keyBindings)
+        while i > 0:
+            if self.keyBindings[i - 1].handler == handler:
+                del self.keyBindings[i - 1]
+            i = i - 1
+
     def hasKeyBinding (self, newKeyBinding, typeOfSearch="strict"):
         """Return True if keyBinding is already in self.keyBindings.
 
            The typeOfSearch can be:
-              "strict":      matches everything (description, modifiers, key)
+              "strict":      matches description, modifiers, key
               "description": matches only description.
               "keys":        matches only modifiers and key.
         """
